var isColoringLayoutShown = false;
var isColoringLayoutReady = false;
var lastColoringImagePath = "";

var canvas = document.getElementById("myCanvas");;
var ctx = canvas.getContext("2d");
canvas.addEventListener('click', onCanvasClicked, false);
canvas.addEventListener("wheel", wheelZoom, false);

var imgData;

var dataRowWidth;
var thresholdAlpha = 90; 

var allPolygonsPixels = []; 
var pixelToPolygon = {};

var imgStartWidth;
var imgStartHeight;
var imageX;
var imageY;
var selectedColor;

var markedR = 180;
var markedG = 130;
var markedB = 80;

var canvasContainer = document.getElementById("canvas-container");

var img = document.getElementById("overlay-image");
img.addEventListener('click', onCanvasClicked, false);
img.addEventListener("wheel", wheelZoom, false);
img.setAttribute('draggable', false);

img.onload = function() {
		if(!isColoringLayoutShown) {
			return;
		}

		setupColoringLayout();
		hideLoadingScreen();
		delayEnableColoring();
	};

function showColoringLayout(catIndex, drawingIndex) {
	var imagePath = "images/drawings/" + categoriesImages[catIndex][0].split('.').slice(0, -1).join('.') + "/" + drawingIndex + ".png"; 
	if(lastColoringImagePath !== imagePath) {
		lastColoringImagePath = imagePath;
		$("#colors-container span").removeClass("selected"); // remove all selected colors

		// setup colors
		setColorOptions(catIndex, drawingIndex);
		delayLoadNewImage();
	} else {
		isColoringLayoutShown = true;
		isColoringLayoutReady = true;
		hideLoadingScreen();
	}	
}

function delayLoadNewImage() {
	setTimeout(function() {
			isColoringLayoutShown = true;
			img.setAttribute("src", lastColoringImagePath);
		}, 100);
}

function setupColoringLayout() {
	var imageSizeOffset = 0; //30
	
	// check what the dimensions of the image on the canvas would beginPath
	var imgWidthScale = canvas.width/(img.naturalWidth + imageSizeOffset);
	var imgHeightScale = canvas.height/(img.naturalHeight + imageSizeOffset);

	if(imgWidthScale < imgHeightScale) {
		imgStartWidth = img.naturalWidth * imgWidthScale;
		imgStartHeight = img.naturalHeight * imgWidthScale;
	} else {
		imgStartWidth = img.naturalWidth * imgHeightScale;
		imgStartHeight = img.naturalHeight * imgHeightScale;
	}
	
	// draw the image on the middle
	imageX = (canvas.width - imgStartWidth)/2;
	imageY = (canvas.height - imgStartHeight )/2;

	zoomSlider.value = 0;
	ctx.clearRect(0, 0, canvas.width, canvas.height);
	ctx.drawImage(img, imageX, imageY, imgStartWidth, imgStartHeight);

    imgData = ctx.getImageData(0, 0, canvas.width, canvas.height);
 	
	dataRowWidth = canvas.width*4;

	createPolygons();
	selectFirstColorElement();
	updateCanvas();
}

function delayEnableColoring() {
	setTimeout(function() {
			isColoringLayoutReady = true;
	}, 100);
}

function createPolygons() {
	allPolygonsPixels = []; 
	pixelToPolygon = {};

	for (var i = 0; i < imgData.data.length; i += 4) {	
		expandPixels(i);
	}
	
	imgData = ctx.getImageData(0, 0, canvas.width, canvas.height);
}

function expandPixels (index) {
	if(imgData.data[index + 3] > thresholdAlpha) {
		return;
	}

	var polygonPixels = [];
	var expandingPixelsIndex = [];
	expandingPixelsIndex.push(index); // pixel on this position is not on the border
	
	var curIndex = 0;

	// check every pixel inside of expandingPixelsIndex array
	while (expandingPixelsIndex.length != 0) {
	    curIndex = expandingPixelsIndex[0];

	     //right
	    if (!checkIfPixelDetermined(imgData.data[curIndex+7]) && getPixelX(curIndex) != canvas.width-1) {
				expandingPixelsIndex.push(curIndex+4);
				colorTargetPixel(curIndex+4);
		   }
	
		// left
		if (!checkIfPixelDetermined(imgData.data[curIndex-1]) && getPixelX(curIndex) != 0) { // if left pixel is not next to the border then put it in the expandingPixelsIndex array
				expandingPixelsIndex.push(curIndex-4);
				colorTargetPixel(curIndex-4);
			}
			
		// bottom
		if (!checkIfPixelDetermined(imgData.data[curIndex+3 + dataRowWidth])) {
				expandingPixelsIndex.push(curIndex + dataRowWidth);
				colorTargetPixel(curIndex + dataRowWidth);
			
			}
		
		// top
		if (!checkIfPixelDetermined(imgData.data[curIndex+3 - dataRowWidth])) {
					expandingPixelsIndex.push(curIndex - dataRowWidth);
					colorTargetPixel(curIndex - dataRowWidth);
		}
			
		colorTargetPixel(curIndex);
		pixelToPolygon[(curIndex / 4).toString()] = allPolygonsPixels.length;
		polygonPixels.push(curIndex);
		expandingPixelsIndex.splice(0, 1);
	}
		
	
	if(polygonPixels.length > 0) {
		allPolygonsPixels.push(polygonPixels);
	}	
}

function getPixelX(index) {
	var y = Math.floor((index/4)/canvas.width);
	var x = index/4 - y*canvas.width;
	return x;
}

function getPixelXY(index) {
	var y = Math.floor((index/4)/canvas.width);
	var x = index/4 - y*canvas.width;
	return [x, y];
}

function checkIfPixelDetermined (a) {
	// check alpha
	if(a == undefined || a >= thresholdAlpha)  {
		//pixel is already here
		return true;
	} else {
		return false;
	}
}

function onCanvasClicked(event) {
	if(!isColoringLayoutReady) {
		return;
	}

	if(totalDragDistance > 11) {
		return;
	}
	
	var rect = canvas.getBoundingClientRect();
	//click position on the canvas
	var x = (event.pageX - rect.left) / cScale / globalScale;
    var y = (event.pageY - rect.top) / cScale / globalScale;

    var index = (Math.floor(y) * canvas.width + Math.floor(x));

	var polygonIndex = pixelToPolygon[index.toString()];
	
	if(polygonIndex == undefined) {// if undefined find the closest one
		var inc = 1;
		var newIndex;
		// try to find the most left or most right one
		while(inc < 4) {
			inc++;
			// try left
			newIndex = index - inc;
			polygonIndex = pixelToPolygon[newIndex.toString()];
			if(polygonIndex != undefined) {
				break;
			}
			
			// try right
			newIndex = index + inc;
			polygonIndex = pixelToPolygon[newIndex.toString()];
			if(polygonIndex != undefined) {
				break;
			} 
			
			// try top
			newIndex = index - canvas.width*inc;
			polygonIndex = pixelToPolygon[newIndex.toString()];
			if(polygonIndex != undefined) {
				break;
			} 
			
			// try bottom
			newIndex = index + canvas.width*inc;
			polygonIndex = pixelToPolygon[newIndex.toString()];
			if(polygonIndex != undefined) {
				break;
			} 
		}
	}

	if(polygonIndex != undefined) {
		colorPolygon(polygonIndex);
	}
}

function colorPolygon(polIndex) {
	var polygonPixels = allPolygonsPixels[polIndex];
		//ctx.fillStyle = "rgba("+selectedColor[0]+","+selectedColor[1]+","+selectedColor[2]+","+(255)+")";

		for(var i = 0; i< polygonPixels.length; i++) {
			paintPixel(polygonPixels[i]);
			//var xy = getPixelXY(polygonPixels[i]);
			//ctx.fillRect( xy[0], xy[1], 1, 1 );
		}

		ctx.putImageData(imgData, 0, 0);
}


// color target pixel with a selected color
function paintPixel (index) {	
		imgData.data[index] = selectedColor[0]; //red	
		imgData.data[index+1] = selectedColor[1]; //green
		imgData.data[index+2] = selectedColor[2]; //blue
		imgData.data[index+3] = 255; // alpha
}

function colorTargetPixel (index) {	
	imgData.data[index] = markedR; //red	
	imgData.data[index+1] = markedG; //green
	imgData.data[index+2] = markedB; //blue
	imgData.data[index+3] = 255; // alpha
}

function setColorOptions(catIndex, drawingIndex) {
	var colorArr = colorsArray[catIndex][drawingIndex-1];
	if(colorArr == null) {
		// error, reverting to the first color set
		console.log("Error! Reverting to the first color set.");
		colorArr = colorsArray[0][0];
	}

	for(var i = 0; i < colorArr.length; i++) {
		document.getElementById("color-option-" + (i+1)).style.backgroundColor = "#" + colorArr[i];
	}
}

function getCurrentTime() {
	return new Date().getTime();
}

function wheelZoom(ev) {
	ev.preventDefault();
	// zoom in or out on mouse wheel
	zoomSlider.value = parseFloat(zoomSlider.value) - event.deltaY * 1.7;
	updateCanvas();
}

/*
* zoom 
*/
var cScale = 1;
var buttonZoomValue = 700;
var zoomSlider = document.getElementById("zoom-slider");
zoomSlider.addEventListener
$("#zoom-slider").on("input change", function() {
	 updateCanvas();
});
	
function zoomCanvas(value) {
	cScale = value/1000;

	// zoom canvas
	canvas.style.width = (canvas.width * cScale * globalScale) + "px";
	canvas.style.height = (canvas.height * cScale * globalScale) + "px";
	if(img != null) {
		img.style.width = (imgStartWidth * cScale * globalScale) + "px";
		img.style.height = (imgStartHeight * cScale * globalScale) + "px";
	}
}

document.getElementById("zoom-out").addEventListener('click', function(e) {
		zoomSlider.value = (parseFloat(zoomSlider.value) - buttonZoomValue);
		updateCanvas();
}, false);

document.getElementById("zoom-in").addEventListener('click', function(e) {
		zoomSlider.value = (parseFloat(zoomSlider.value) + buttonZoomValue);
		updateCanvas();
}, false);

// updates canvas zoom and pan position
function updateCanvas() {
	var prevWidth = canvas.offsetWidth;
	var prevHeight = canvas.offsetHeight;
	zoomCanvas(zoomSlider.value);
	panCanvas((prevWidth - canvas.offsetWidth)/2 ,
		(prevHeight - canvas.offsetHeight)/2);
}
/*
* touch and mouse events
*/
canvasContainer.addEventListener("click", function(ev) {
	ev.preventDefault();
});

var dragStartX;
var dragStartY;

var lastDeltaX = 0;
var lastDeltaY = 0;

var prevX;
var prevY;

var prevZoomDiff = -1;
var dragLocked = false;

var isDragging = false;
var totalDragDistance = 0;

canvasContainer.addEventListener('mousedown', function(e) {
	totalDragDistance = 0;
	dragStart(e.clientX, e.clientY);
}, false);

canvasContainer.addEventListener('mousemove', function(e) {
	e.preventDefault();
}, false);

document.addEventListener('mousemove', function(e) {
		//e.preventDefault();
		drag(e.clientX, e.clientY);
}, false);

document.addEventListener('mouseup', function(e) {
	dragEnd(e.clientX, e.clientY);
}, false);


canvasContainer.addEventListener('touchstart', function(e) {
	totalDragDistance = 0;
	dragStart(e.touches[0].clientX, e.touches[0].clientY);
	resetTouchZoomParams();
}, false);

canvasContainer.addEventListener('touchmove', function(e) {
	e.preventDefault();
	if(e.touches.length == 1 ) {
		resetTouchZoomParams();
		drag(e.changedTouches[0].clientX, e.changedTouches[0].clientY);
	} else if(e.touches.length == 2){ // if more than one touch then zoom it
		dragLocked = true;
		//var newDiff =  Math.abs(e.changedTouches[0].clientX - e.changedTouches[1].clientX); 
		var dx = e.changedTouches[0].clientX - e.changedTouches[1].clientX;
		var dy = e.changedTouches[0].clientY - e.changedTouches[1].clientY;
		
		var newDiff = Math.sqrt(dx*dx + dy*dy);
		if(prevZoomDiff != -1) { // this needs to be setup
			var zoomDif = newDiff - prevZoomDiff;
			zoomSlider.value = (parseFloat(zoomSlider.value) + zoomDif*9);
			updateCanvas();
		} 
		
		prevZoomDiff = newDiff;
	} else { // if more than 2 touches
		dragLocked = true;
		resetTouchZoomParams();
	}
}, false);

canvasContainer.addEventListener('touchend', function(e) {
	if(e.touches.length == 0 ) {
		dragEnd(e.changedTouches[0].clientX, e.changedTouches[0].clientY);
	}
	resetTouchZoomParams();
}, false);

function resetTouchZoomParams() {
	prevZoomDiff = -1;
}

function dragStart(x, y) {
	if(isDragging) { // prevent second touch
			return;
	}
	
	// reset the drag locked indicator
	dragLocked = false;
	
	prevX = x;
    prevY = y;
	isDragging = true;
}

function drag(x, y) {
	if(!isDragging || dragLocked) {
			return;
	}
	
	var deltaX, deltaY;
	deltaX = x - prevX;
    deltaY = y - prevY;
	panCanvas(deltaX,deltaY);
	prevX = x;
	prevY = y;
	
	// add to totalDragDistance
	totalDragDistance += Math.abs(deltaX) + Math.abs(deltaY);
}

function dragEnd(x, y) {
	if(!isDragging) {
			return;
	}
	
	lastDeltaX = x - dragStartX + lastDeltaX;
    lastDeltaY = y - dragStartY + lastDeltaY;
	isDragging = false;
}


var lastLeft = 0;
var lastTop = 0;
// move canvas content
function panCanvas(deltaX, deltaY) {
		lastLeft += deltaX;
		lastTop += deltaY;
	
		if(lastLeft > 0) {
			lastLeft = 0;
		} else if(canvas.offsetWidth + lastLeft < canvas.offsetWidth/cScale ) {
			lastLeft = -(canvas.offsetWidth - canvas.offsetWidth/cScale);
		}	
		
		if(lastTop > 0) {
			lastTop = 0;
		} else if(canvas.offsetHeight + lastTop < canvas.offsetHeight/cScale ) {
			lastTop = -(canvas.offsetHeight - canvas.offsetHeight/cScale);
		}
		
		canvas.style.left = lastLeft + "px";
		canvas.style.top = lastTop + "px";
		if(img != null) {
			img.style.left = (lastLeft + imageX*cScale*globalScale) + "px";
			img.style.top = (lastTop + imageY*cScale*globalScale) + "px";	
		}
}

/*
* Colors
*****************************************************************
*/
// set selected color on the first one
$("#colors-container span").on("click", function() {
	$("#colors-container span").removeClass("selected");
	 $(this).addClass("selected");
	 selectedColor = getColorArray($(this)[0].style.backgroundColor);
});

function selectFirstColorElement() {
	$("#colors-container span").first().addClass("selected");
	selectedColor = getColorArray($("#colors-container span")[0].style.backgroundColor);
}

function getColorArray(rgbString) {
	return rgbString.substring(4, rgbString.length-1)
         .replace(/ /g, '')
         .split(',');
}

// back button
document.getElementById("back-button-drawing").addEventListener("click", function(ev) {
	isColoringLayoutShown = false;
	isColoringLayoutReady = false;
	showChooseDrawingLayout();
});

// restart coloring button
document.getElementById("back-restart-drawing").addEventListener("click", function(ev) {
	ctx.clearRect(0, 0, canvas.width, canvas.height);
	ctx.drawImage(img, imageX, imageY, imgStartWidth, imgStartHeight);
	imgData = ctx.getImageData(0, 0, canvas.width, canvas.height);
});





